"use strict";
const graphlib = require("graphlib");
const typed_sql_1 = require("typed-sql");
const mapping_1 = require("typed-sql/lib/mapping");
const common_1 = require("./common");
class SqlGraph {
    constructor() {
        this.graph = new graphlib.Graph();
    }
    addMasterDetailJoin(masterMapping, detailMapping, joinCondition) {
        let masterTableName = typed_sql_1.WrappedMappingData.getMappingData(masterMapping).getTableName();
        let detailTableName = typed_sql_1.WrappedMappingData.getMappingData(detailMapping).getTableName();
        this.graph.setNode(masterTableName, masterMapping);
        this.graph.setNode(detailTableName, detailMapping);
        this.graph.setEdge(masterTableName, detailTableName, joinCondition);
        let join = {
            masterMapping,
            detailMapping,
            joinCondition
        };
        return join;
    }
    addSimpleMasterDetailJoin(masterAttribute, detailAttribute) {
        return this.addMasterDetailJoin(mapping_1.Attribute.getBaseAttribute(masterAttribute).mapping, mapping_1.Attribute.getBaseAttribute(detailAttribute).mapping, typed_sql_1.equal(masterAttribute, detailAttribute));
    }
    addJoin(join) {
        this.addMasterDetailJoin(join.masterMapping, join.detailMapping, join.joinCondition);
    }
    /**
     * Return a list of joins linking the source relation to the target.
     */
    getJoinPath(sourceMapping, targetMapping) {
        let sourceTableName = typed_sql_1.WrappedMappingData.getMappingData(sourceMapping).getTableName();
        let targetTableName = typed_sql_1.WrappedMappingData.getMappingData(targetMapping).getTableName();
        let edgeData = graphlib.alg.dijkstra(this.graph, sourceTableName);
        let edgeTargetNode = targetTableName;
        let joinPath = [];
        while (edgeTargetNode !== sourceTableName) {
            let edge = edgeData[edgeTargetNode];
            let edgeSourceNode = edge.predecessor;
            if (!edgeSourceNode) {
                throw new SqlGraphTopologyError("No join path found.");
            }
            joinPath.push({
                masterMapping: this.getNodeMapping(edgeSourceNode),
                detailMapping: this.getNodeMapping(edgeTargetNode),
                joinCondition: this.getEdgeJoinCondition(edgeSourceNode, edgeTargetNode)
            });
            edgeTargetNode = edgeSourceNode;
        }
        return joinPath.reverse();
    }
    getNodeMapping(tableName) {
        return this.graph.node(tableName);
    }
    getEdgeJoinCondition(sourceTableName, targetTableName) {
        return this.graph.edge(sourceTableName, targetTableName);
    }
}
exports.SqlGraph = SqlGraph;
class SqlGraphTopologyError extends common_1.CustomError {
}
exports.SqlGraphTopologyError = SqlGraphTopologyError;
//# sourceMappingURL=sql_graph.js.map